package com.landray.kmss.sys.attachment.integrate.wps.util;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTRunTrackChange;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTBookmark;
import java.util.Collection;
import java.util.Iterator;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import java.util.List;
import java.util.LinkedHashMap;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import java.util.HashMap;

public class BookMarks
{
    private HashMap<String, BookMark> _bookmarks;
    
    public BookMarks(final XWPFDocument document) {
        this._bookmarks = null;
        this._bookmarks = new LinkedHashMap<String, BookMark>();
        this.procParaList(document.getParagraphs());
        final List<XWPFTable> tableList = document.getTables();
        for (final XWPFTable table : tableList) {
            final List<XWPFTableRow> rowList = table.getRows();
            for (final XWPFTableRow row : rowList) {
                final List<XWPFTableCell> cellList = row.getTableCells();
                for (final XWPFTableCell cell : cellList) {
                    this.procParaList(cell);
                }
            }
        }
    }
    
    public BookMark getBookmark(final String bookmarkName) {
        BookMark bookmark = null;
        if (this._bookmarks.containsKey(bookmarkName)) {
            bookmark = this._bookmarks.get(bookmarkName);
        }
        return bookmark;
    }
    
    public Collection<BookMark> getBookmarkList() {
        return this._bookmarks.values();
    }
    
    public Iterator<String> getNameIterator() {
        return this._bookmarks.keySet().iterator();
    }
    
    private void procParaList(final XWPFTableCell cell) {
        final List<XWPFParagraph> paragraphList = cell.getParagraphs();
        for (final XWPFParagraph paragraph : paragraphList) {
            final List<CTBookmark> bookmarkList = paragraph.getCTP().getBookmarkStartList();
            for (final CTBookmark bookmark : bookmarkList) {
                this._bookmarks.put(bookmark.getName(), new BookMark(bookmark, paragraph, cell, "0"));
            }
            final List<CTRunTrackChange> delList = paragraph.getCTP().getDelList();
            for (final CTRunTrackChange del : delList) {
                final List<CTBookmark> trackBookMarkStartList = del.getBookmarkStartList();
                for (final CTBookmark bookmark2 : trackBookMarkStartList) {
                    this._bookmarks.put(bookmark2.getName(), new BookMark(bookmark2, paragraph, cell, "1"));
                }
            }
            final List<CTRunTrackChange> insList = paragraph.getCTP().getInsList();
            for (final CTRunTrackChange ins : insList) {
                final List<CTBookmark> trackBookMarkStartList2 = ins.getBookmarkStartList();
                for (final CTBookmark bookmark3 : trackBookMarkStartList2) {
                    this._bookmarks.put(bookmark3.getName(), new BookMark(bookmark3, paragraph, cell, "2"));
                }
            }
        }
    }
    
    private void procParaList(final List<XWPFParagraph> paragraphList, final XWPFTableRow tableRow) {
        NamedNodeMap attributes = null;
        Node colFirstNode = null;
        Node colLastNode = null;
        int firstColIndex = 0;
        int lastColIndex = 0;
        for (final XWPFParagraph paragraph : paragraphList) {
            final List<CTBookmark> bookmarkList = paragraph.getCTP().getBookmarkStartList();
            for (final CTBookmark bookmark : bookmarkList) {
                attributes = bookmark.getDomNode().getAttributes();
                if (attributes != null) {
                    colFirstNode = attributes.getNamedItem("w:colFirst");
                    colLastNode = attributes.getNamedItem("w:colLast");
                    if (colFirstNode != null && colLastNode != null) {
                        firstColIndex = Integer.parseInt(colFirstNode.getNodeValue());
                        lastColIndex = Integer.parseInt(colLastNode.getNodeValue());
                        if (firstColIndex == lastColIndex) {
                            this._bookmarks.put(bookmark.getName(), new BookMark(bookmark, paragraph, tableRow.getCell(firstColIndex), "0"));
                        }
                        else {
                            System.out.println("This bookmark " + bookmark.getName() + " identifies a number of cells in the " + "table. That condition is not handled yet.");
                        }
                    }
                    else {
                        this._bookmarks.put(bookmark.getName(), new BookMark(bookmark, paragraph, tableRow.getCell(1), "0"));
                    }
                }
                else {
                    this._bookmarks.put(bookmark.getName(), new BookMark(bookmark, paragraph, tableRow.getCell(1), "0"));
                }
            }
        }
    }
    
    private void procParaList(final List<XWPFParagraph> paragraphList) {
        for (final XWPFParagraph paragraph : paragraphList) {
            final List<CTBookmark> bookmarkList = paragraph.getCTP().getBookmarkStartList();
            for (final CTBookmark bookmark : bookmarkList) {
                this._bookmarks.put(bookmark.getName(), new BookMark(bookmark, paragraph, "0"));
            }
            final List<CTRunTrackChange> delList = paragraph.getCTP().getDelList();
            for (final CTRunTrackChange del : delList) {
                final List<CTBookmark> trackBookMarkStartList = del.getBookmarkStartList();
                for (final CTBookmark bookmark2 : trackBookMarkStartList) {
                    this._bookmarks.put(bookmark2.getName(), new BookMark(bookmark2, paragraph, "1"));
                }
            }
            final List<CTRunTrackChange> insList = paragraph.getCTP().getInsList();
            for (final CTRunTrackChange inl : insList) {
                final List<CTBookmark> trackBookMarkStartList2 = inl.getBookmarkStartList();
                for (final CTBookmark bookmark3 : trackBookMarkStartList2) {
                    this._bookmarks.put(bookmark3.getName(), new BookMark(bookmark3, paragraph, "2"));
                }
            }
        }
    }
}
